#include <stdio.h>

#include <Application.h>
#include <File.h>
#include <MediaFile.h>
#include <MediaTrack.h>
#include <Bitmap.h>

BMediaTrack *GetVideoTrack(BMediaFile &mediaFile, media_format &mf)
{
	for (int32 i = 0; i < mediaFile.CountTracks(); i++) {
		BMediaTrack *track = mediaFile.TrackAt(i);

		if (track->EncodedFormat(&mf) >= B_OK) {
			switch (mf.type) {
			case B_MEDIA_RAW_VIDEO:
			case B_MEDIA_ENCODED_VIDEO:
				return track;
			}
			mediaFile.ReleaseTrack(track);
		}
	}
	return NULL;
}

int main(int argCnt, char **args)
{
	BApplication app("application/x-vnd.Test-App");
	if (argCnt < 2) return 1;
	BFile file(args[1], B_READ_ONLY);
	if (file.InitCheck() < B_OK) return 1;
	BMediaFile mediaFile(&file);
	media_format mf;
	if (mediaFile.InitCheck() < B_OK) return 1;
	BMediaTrack *videoTrack = GetVideoTrack(mediaFile, mf);
	if (videoTrack == NULL) return 1;

	BRect bitmapBounds(
		0.0,
		0.0,
		mf.u.encoded_video.output.display.line_width - 1.0,
		mf.u.encoded_video.output.display.line_count - 1.0
	);

	videoTrack->DecodedFormat(&mf);
	BBitmap bitmap(bitmapBounds, mf.u.raw_video.display.format);
	if (bitmap.InitCheck() < B_OK) return 1;

	for (;;) {
		int64 frameCnt = 0;
		media_header mh;
		status_t res = videoTrack->ReadFrames(bitmap.Bits(), &frameCnt, &mh);
		printf("frameCnt: %lld, mh.start_time: %lld\n", frameCnt, mh.start_time);
		if (res < B_OK)
			printf("res: %x\n", res);
		if (frameCnt <= 0 || res < B_OK)
			break;
	}

	mediaFile.ReleaseTrack(videoTrack); videoTrack = NULL;

	return 0;
}
